@@ -75,16 +75,18 @@ used in the database layer.
 sub inflate_column {
   my ($self, $col, $attrs) = @_;
 
+  my $colinfo = $self->column_info($col);
+
   $self->throw_exception("InflateColumn does not work with FilterColumn")
     if $self->isa('DBIx::Class::FilterColumn') &&
-      defined $self->column_info($col)->{_filter_info};
+      defined $colinfo->{_filter_info};
 
   $self->throw_exception("No such column $col to inflate")
     unless $self->has_column($col);
   $self->throw_exception("inflate_column needs attr hashref")
     unless ref $attrs eq 'HASH';
-  $self->column_info($col)->{_inflate_info} = $attrs;
-  my $acc = $self->column_info($col)->{accessor};
+  $colinfo->{_inflate_info} = $attrs;
+  my $acc = $colinfo->{accessor};
   $self->mk_group_accessors('inflated_column' => [ (defined $acc ? $acc : $col), $col]);
   return 1;
 }
