@@ -109,88 +109,87 @@ the C<datetime_undef_if_invalid> option in the column info:
 
 sub register_column {
   my ($self, $column, $info, @rest) = @_;
+
   $self->next::method($column, $info, @rest);
-  return unless defined($info->{data_type});
 
-  my $type;
-
-  for (qw/date datetime timestamp/) {
+  my $requested_type;
+  for (qw/datetime timestamp date/) {
     my $key = "inflate_${_}";
+    if (exists $info->{$key}) {
 
-    next unless exists $info->{$key};
-    return unless $info->{$key};
+      # this bailout is intentional
+      return unless $info->{$key};
 
-    $type = $_;
-    last;
-  }
-
-  unless ($type) {
-    $type = lc($info->{data_type});
-    if ($type eq "timestamp with time zone" || $type eq "timestamptz") {
-      $type = "timestamp";
-      $info->{_ic_dt_method} ||= "timestamp_with_timezone";
-    } elsif ($type eq "timestamp without time zone") {
-      $type = "timestamp";
-      $info->{_ic_dt_method} ||= "timestamp_without_timezone";
-    } elsif ($type eq "smalldatetime") {
-      $type = "datetime";
-      $info->{_ic_dt_method} ||= "smalldatetime";
+      $requested_type = $_;
+      last;
     }
   }
 
-  if ( defined $info->{extra}{timezone} ) {
-    carp "Putting timezone into extra => { timezone => '...' } has been deprecated, ".
-         "please put it directly into the '$column' column definition.";
-    $info->{timezone} = $info->{extra}{timezone} unless defined $info->{timezone};
+  return if (!$requested_type and !$info->{data_type});
+
+  my $data_type = lc( $info->{data_type} || '' );
+
+  # _ic_dt_method will follow whatever the registration requests
+  # thus = instead of ||=
+  if ($data_type eq 'timestamp with time zone' || $data_type eq 'timestamptz') {
+    $info->{_ic_dt_method} = 'timestamp_with_timezone';
+  }
+  elsif ($data_type eq 'timestamp without time zone') {
+    $info->{_ic_dt_method} = 'timestamp_without_timezone';
+  }
+  elsif ($data_type eq 'smalldatetime') {
+    $info->{_ic_dt_method} = 'smalldatetime';
+  }
+  elsif ($data_type =~ /^ (?: date | datetime | timestamp ) $/x) {
+    $info->{_ic_dt_method} = $data_type;
+  }
+  elsif ($requested_type) {
+    $info->{_ic_dt_method} = $requested_type;
+  }
+  else {
+    return;
   }
 
-  if ( defined $info->{extra}{locale} ) {
-    carp "Putting locale into extra => { locale => '...' } has been deprecated, ".
-         "please put it directly into the '$column' column definition.";
-    $info->{locale} = $info->{extra}{locale} unless defined $info->{locale};
-  }
-
-  my $undef_if_invalid = $info->{datetime_undef_if_invalid};
-
-  if ($type eq 'datetime' || $type eq 'date' || $type eq 'timestamp') {
-    # This shallow copy of %info avoids t/52_cycle.t treating
-    # the resulting deflator as a circular reference.
-    my %info = ( '_ic_dt_method' => $type , %{ $info } );
-
-    if (defined $info->{extra}{floating_tz_ok}) {
-      carp "Putting floating_tz_ok into extra => { floating_tz_ok => 1 } has been deprecated, ".
-           "please put it directly into the '$column' column definition.";
-      $info{floating_tz_ok} = $info->{extra}{floating_tz_ok};
+  if ($info->{extra}) {
+    for my $slot (qw/timezone locale floating_tz_ok/) {
+      if ( defined $info->{extra}{$slot} ) {
+        carp "Putting $slot into extra => { $slot => '...' } has been deprecated, ".
+             "please put it directly into the '$column' column definition.";
+        $info->{$slot} = $info->{extra}{$slot} unless defined $info->{$slot};
+      }
     }
-
-    $self->inflate_column(
-      $column =>
-        {
-          inflate => sub {
-            my ($value, $obj) = @_;
-
-            my $dt = try
-              { $obj->_inflate_to_datetime( $value, \%info ) }
-              catch {
-                $self->throw_exception ("Error while inflating ${value} for ${column} on ${self}: $_")
-                  unless $undef_if_invalid;
-                undef;  # rv
-              };
-
-            return (defined $dt)
-              ? $obj->_post_inflate_datetime( $dt, \%info )
-              : undef
-            ;
-          },
-          deflate => sub {
-            my ($value, $obj) = @_;
-
-            $value = $obj->_pre_deflate_datetime( $value, \%info );
-            $obj->_deflate_from_datetime( $value, \%info );
-          },
-        }
-    );
   }
+
+  # shallow copy to avoid unfounded(?) Devel::Cycle complaints
+  my $infcopy = {%$info};
+
+  $self->inflate_column(
+    $column =>
+      {
+        inflate => sub {
+          my ($value, $obj) = @_;
+
+          my $dt = try
+            { $obj->_inflate_to_datetime( $value, $infcopy ) }
+            catch {
+              $self->throw_exception ("Error while inflating ${value} for ${column} on ${self}: $_")
+                unless $infcopy->{datetime_undef_if_invalid};
+              undef;  # rv
+            };
+
+          return (defined $dt)
+            ? $obj->_post_inflate_datetime( $dt, $infcopy )
+            : undef
+          ;
+        },
+        deflate => sub {
+          my ($value, $obj) = @_;
+
+          $value = $obj->_pre_deflate_datetime( $value, $infcopy );
+          $obj->_deflate_from_datetime( $value, $infcopy );
+        },
+      }
+  );
 }
 
 sub _flate_or_fallback
