@@ -202,7 +202,6 @@ SELECT $out_sel FROM (
 
 EOS
 
-  $sql =~ s/\s*\n\s*/ /g;   # easier to read in the debugger
   return $sql;
 }
 
@@ -307,7 +306,6 @@ EOS
 EOS
   }
 
-  $sql =~ s/\s*\n\s*/ /g;   # easier to read in the debugger
   return $sql;
 }
 
@@ -390,9 +388,9 @@ sub _Top {
 
     # since whatever order bindvals there are, they will be realiased
     # and need to show up in front of the entire initial inner subquery
-    # Unshift *from_bind* to make this happen (horrible, horrible, but
-    # we don't have another mechanism yet)
-    unshift @{$self->{from_bind}}, @{$self->{order_bind}};
+    # *unshift* the selector bind stack to make this happen (horrible,
+    # horrible, but we don't have another mechanism yet)
+    unshift @{$self->{select_bind}}, @{$self->{order_bind}};
   }
 
   # and this is order re-alias magic
@@ -433,7 +431,6 @@ sub _Top {
     $order_by_requested,
   ) if ( ($offset && $order_by_requested) || ($mid_sel ne $out_sel) );
 
-  $sql =~ s/\s*\n\s*/ /g;   # easier to read in the debugger
   return $sql;
 }
 
@@ -579,7 +576,6 @@ EOS
     ,
   );
 
-  $sql =~ s/\s*\n\s*/ /g;   # easier to read in the debugger
   return $sql;
 }
 
