@@ -32,18 +32,20 @@ __PACKAGE__->add_columns(
         size      => 100,
         is_nullable => 1,
     },
+    encoded => {
+        data_type => 'integer',
+        is_nullable => 1,
+    },
 );
 
 __PACKAGE__->set_primary_key('employee_id');
 __PACKAGE__->position_column('position');
 
-#__PACKAGE__->add_unique_constraint(position_group => [ qw/position group_id/ ]);
+# Do not add unique constraints here - different groups are used throughout
+# the ordered tests
 
-__PACKAGE__->mk_classdata('field_name_for', {
-    employee_id => 'primary key',
-    position    => 'list position',
-    group_id    => 'collection column',
-    name        => 'employee name',
+__PACKAGE__->belongs_to (secretkey => 'DBICTest::Schema::Encoded', 'encoded', {
+  join_type => 'left'
 });
 
 1;
