@@ -1,6 +1,7 @@
-package DBICTest::Schema::SelfRefAlias;
+package # hide from PAUSE 
+    DBICTest::Schema::SelfRefAlias;
 
-use base 'DBIx::Class::Core';
+use base qw/DBICTest::BaseResult/;
 
 __PACKAGE__->table('self_ref_alias');
 __PACKAGE__->add_columns(
@@ -13,4 +14,7 @@ __PACKAGE__->add_columns(
 );
 __PACKAGE__->set_primary_key(qw/self_ref alias/);
 
+__PACKAGE__->belongs_to( self_ref => 'DBICTest::Schema::SelfRef' );
+__PACKAGE__->belongs_to( alias => 'DBICTest::Schema::SelfRef' );
+
 1;
