@@ -1,9 +1,6 @@
-package # hide from PAUSE
+package # hide from PAUSE 
     DBICTest::Schema::TimestampPrimaryKey;
 
-use warnings;
-use strict;
-
 use base qw/DBICTest::BaseResult/;
 
 __PACKAGE__->table('timestamp_primary_key_test');
